unit uModemComm;

interface

uses
  uSerialComm, uQueue,
  Winapi.Windows, Winapi.Messages, System.Classes;

type
  TOpenPortEvent = procedure(Sender: TObject; const AReturnCode: Integer) of object;

  TModemComm = class(TThread)
  private
    FSerialComm    : TSerialComm;
    FOnInitialize  : TNotifyEvent;
    FOnOpenPort    : TOpenPortEvent;
    FMessageHandler: HWND;
    FQueue         : TQueue;
    FLastResponse  : String;
    FWaitEvent     : THandle;
    FReturnCode    : Integer;

    procedure InternalMessageHandler(var AMessage: TMessage);
    procedure Signal;

    procedure syncOpenPort;
    procedure syncInitialize;

  protected
    procedure Execute; override;

  public
    constructor Create;
    destructor Destroy; override;

    procedure SetScript(const AFile: String);
    procedure ClearQueue;

    procedure OpenPort(const APort: String);
    procedure Initialize;

    procedure Shutdown;

    property Serial      : TSerialComm read FSerialComm;

    property OnInitialize: TNotifyEvent read FOnInitialize write FOnInitialize;
    property OnOpenPort  : TOpenPortEvent read FOnOpenPort write FOnOpenPort;

  end;

implementation

{ TModemComm }

const
  CR = #13;


constructor TModemComm.Create;
begin
  inherited Create(TRUE);

  FMessageHandler := AllocateHWnd(InternalMessageHandler);

  FWaitEvent := CreateEvent(nil, FALSE, FALSE, nil);

  FQueue := TQueue.Create;

  FSerialComm := TSerialComm.Create;
  FSerialComm.SetNotifyHandle(FMessageHandler);
end;

destructor TModemComm.Destroy;
begin
  FSerialComm.Free;
  FQueue.Free;
  CloseHandle(FWaitEvent);
  DeallocateHWnd(FMessageHandler);

  inherited;
end;

procedure TModemComm.Execute;
var
  queue_item: TQueueItem;
begin
  while not Terminated do
  begin
    if FQueue.Count > 0 then
    begin
      queue_item := FQueue[0];

      case queue_item.Event of
        qeOpenPort  : begin
                        FReturnCode := FSerialComm.Start(queue_item.Port);
                        if (not Terminated) and
                           (Assigned(FOnOpenPort)) then
                          Synchronize(syncOpenPort);
                      end;

        qeCommand   : if queue_item.Command <> '' then
                      begin
                        FLastResponse := '';
                        FSerialComm.Write(queue_item.Command + CR);

                        while (FLastResponse = '') and
                              (WaitForSingleObject(FWaitEvent, 0) <> WAIT_OBJECT_0) do
                          Sleep(5);

                        if Pos(queue_item.Response, FLastResponse) > 0 then;
                      end;

        qeInitialize: if Assigned(FOnInitialize) then
                        Synchronize(syncInitialize);
      end;

      if FQueue.Count > 0 then
        FQueue.Delete(0);
    end;

    Sleep(10);
  end;
end;

procedure TModemComm.InternalMessageHandler(var AMessage: TMessage);
begin
  if AMessage.Msg = FSerialComm.WMDataReceive then
    FLastResponse := FSerialComm.Read;

  DefWindowProc(FMessageHandler, AMessage.Msg, AMessage.WParam, AMessage.LParam);
end;

procedure TModemComm.ClearQueue;
begin
  FQueue.Clear;
  Signal;
end;

procedure TModemComm.SetScript(const AFile: String);
begin

end;

procedure TModemComm.Shutdown;
begin
  Terminate;
  Signal;
  WaitFor;
end;

procedure TModemComm.Signal;
begin
  SetEvent(FWaitEvent);
end;

procedure TModemComm.syncInitialize;
begin
  FOnInitialize(self);
end;

procedure TModemComm.syncOpenPort;
begin
  FOnOpenPort(self, FReturnCode);
end;

procedure TModemComm.OpenPort(const APort: String);
begin
  FQueue.Add(TQueueItem.Create(qeOpenPort, APort));
end;

procedure TModemComm.Initialize;
begin
  FQueue.Add(TQueueItem.Create(qeCommand, 'AT; OK'));
  FQueue.Add(TQueueItem.Create(qeCommand, 'AT+CPIN?; +CPIN: READY'));
  FQueue.Add(TQueueItem.Create(qeCommand, 'AT+CMGF=1; OK'));
  FQueue.Add(TQueueItem.Create(qeInitialize));
end;



end.
