unit uSerialComm;

interface

uses
  uSerialCommReader,
  Winapi.Windows, Winapi.Messages, System.Classes;

type
  TSerialComm = class
  private
    const
      READ_BUFFER_SIZE = 4096;

    var
      FPort          : String;
      FHandle        : THandle;
      FNotifyHandle  : THandle;
      FReader        : TSerialCommReader;
      FWMDataReceive : UINT;
      FMessageHandler: HWND;

    function Open: Integer;
    procedure Close;
    function Configure(const ABaud: Integer; const AParity: String; const AData, AStop: Integer): Integer;
    function IsActive: Boolean;
    procedure InternalMessageHandler(var AMessage: TMessage);

  public
    constructor Create;
    destructor Destroy; override;

    function Start(const APort: String): Integer;
    function Write(const AData: String): Integer;
    function Read: String;
    procedure Stop;

    procedure SetNotifyHandle(const ANotifyHandle: THandle);

    property Port         : String read FPort;
    property Handle       : THandle read FHandle;
    property Active       : Boolean read IsActive;
    property WMDataReceive: UINT read FWMDataReceive;
  end;

implementation

uses
  System.SysUtils;



constructor TSerialComm.Create;
begin
  FPort := '';
  FHandle := 0;

  FMessageHandler := AllocateHWnd(InternalMessageHandler);
  FWMDataReceive := RegisterWindowMessage('SkypeDiverter-SerialComm-DataReceive');
end;

destructor TSerialComm.Destroy;
begin
  if Active then
    Stop;

  DeallocateHWnd(FMessageHandler);

  inherited;
end;

procedure TSerialComm.SetNotifyHandle(const ANotifyHandle: THandle);
begin
  FNotifyHandle := ANotifyHandle;
end;

procedure TSerialComm.InternalMessageHandler(var AMessage: TMessage);
begin
  PostMessage(FNotifyHandle, AMessage.Msg, AMessage.WParam, AMessage.LParam);

  DefWindowProc(FMessageHandler, AMessage.Msg, AMessage.WParam, AMessage.LParam);
end;

function TSerialComm.IsActive: Boolean;
begin
  result := (FHandle > 0) and (FHandle <> INVALID_HANDLE_VALUE);
end;

function TSerialComm.Open: Integer;
begin
  FHandle := CreateFile(PChar(Format('\\.\%s', [FPort])), GENERIC_READ or GENERIC_WRITE, 0, nil, OPEN_EXISTING, 0, 0);
  if FHandle = INVALID_HANDLE_VALUE then
    Exit(GetLastError);
  Exit(ERROR_SUCCESS);
end;

procedure TSerialComm.Close;
begin
  CloseHandle(FHandle);
  FHandle := 0;
end;

function TSerialComm.Start(const APort: String): Integer;
var
  res: Integer;
begin
  FPort := APort;

  res := Open;
  if res <> ERROR_SUCCESS then
    Exit(res);

  res := Configure(115200, 'N', 8, 1);
  if res <> ERROR_SUCCESS then
  begin
    CloseHandle(FHandle);
    FHandle := 0;
    Exit(res);
  end;

  FReader := TSerialCommReader.Create(FHandle, READ_BUFFER_SIZE, FMessageHandler, FWMDataReceive);
  FReader.Start;

  Exit(ERROR_SUCCESS);
end;

procedure TSerialComm.Stop;
begin
  FReader.Shutdown;
  FreeAndNil(FReader);
  Close;
end;

function TSerialComm.Configure(const ABaud: Integer; const AParity: String; const AData, AStop: Integer): Integer;
const
  BUFFER_SECONDS = 2;
var
  dcb, dcbp   : TDCB;
  mult        : Integer;
  commtimeouts: TCommTimeouts;
begin
  if not GetCommState(FHandle, dcb) then
    Exit(GetLastError);

  if not BuildCommDCB(PChar(Format('baud=%d parity=%s data=%d stop=%d', [ABaud, AParity, AData, AStop])), dcbp) then
    Exit(GetLastError);

  dcb.BaudRate := dcbp.BaudRate;
  dcb.ByteSize := dcbp.ByteSize;
  dcb.Parity := dcbp.Parity;
  dcb.StopBits := dcbp.StopBits;
  if not SetCommState(FHandle, dcb) then
    Exit(GetLastError);

  mult := Trunc(dcbp.BaudRate / 8 * BUFFER_SECONDS / READ_BUFFER_SIZE);
  if not SetupComm(FHandle, READ_BUFFER_SIZE * mult, READ_BUFFER_SIZE) then
    Exit(GetLastError);

  if not GetCommTimeouts(FHandle, commtimeouts) then
    Exit(GetLastError);

  commtimeouts.ReadIntervalTimeout := MAXDWORD;
  commtimeouts.ReadTotalTimeoutMultiplier := 0;
  commtimeouts.ReadTotalTimeoutConstant := 0;

  if not SetCommTimeouts(FHandle, commtimeouts) then
    Exit(GetLastError);

  Exit(ERROR_SUCCESS);
end;

function TSerialComm.Write(const AData: String): Integer;
const
  BUFFER_SIZE = 4096;
var
  buffer  : array[0..BUFFER_SIZE - 1] of AnsiChar;
  bWritten: DWORD;
begin
  StrPCopy(@buffer[0], AnsiString(AData));
  if not WriteFile(FHandle, buffer, Length(AData), bWritten, nil) then
    Exit(GetLastError);
  Exit(ERROR_SUCCESS);
end;

function TSerialComm.Read: String;
begin
  result := FReader.GetData;
end;


end.
